/******************************************************************************
** Debug function to show the difference between two strings.
******************************************************************************/
function printStringDiff(str1, str2)
{
	// Check our params
	if(!str1)
	{
		print("printStringDiff: missing string parameters");
		return;
	}
	if(!str2)
	{
		print("printStringDiff: missing string 2 parameter");
		return;
	}
	
	var length = str1.length;
	for(var i = 0; i < length; i++)
	{
		if(i > str2.length)
		{
			print("printStringDiff: string 1 the same, but longer than string 2:");
			print("  1)" + str1.substr(i, i+50));
			return;
		}
		
		if(str1.charCodeAt(i) != str2.charCodeAt(i))
		{
			print("printStringDiff: strings are different at point " + i + ":");
			if(i > 10)
			{
				print("  1)" + str1.substring(i-10, i+40));
				print("  2)" + str2.substring(i-10, i+40));
			}
			else
			{
				print("  1)" + str1.substring(i, i+50));
				print("  2)" + str2.substring(i, i+50));
			}
			return;
		}
	}
	
	// If we get here, we've compared the entire length of string 1 and it has
	// matched string 2
	if(str2.length != str1.length)
	{
		print("printStringDiff: string 1 the same, but shorter than string 2:");
		print("  2)" + str2.substr(i, i+50));
		return;
	}
	
	// Otherwise we have a perfect match!
	print("printStringDiff: string 1 perfectly matches string 2");
}

function printListItems()
{
	print("printListItems: categoryIdx = " + categoryIdx);
	var i;
	for(i = 0; i < 10; i++)
	{
		print("listItems ["+categoryIdx+"]["+i+"] is:");
		print("   " +  listItems[categoryIdx][i] );
	}
}

/******************************************************************************
** Pull the data from iTunes and parse it.  The categories and listItems arrays
** will be rebuilt from scratch by this function.
******************************************************************************/
function updateList()
{
//    runCommandInBg("sleep 2; curl -m 10 -s '" +  url + "'", "amazonResponse");

	// First fetch the list data from iTunes
	var url = new URL();
	var destination = "http://ax.phobos.apple.com.edgesuite.net/WebObjects/MZStore.woa/wpa/MRSS/";
	if(preferences.searchType.value == "Albums")
		destination += "topalbums/";
	else
		destination += "topsongs/";
	destination += "sf=143441/";
	if(genres != 0)
		destination += "genre=" + genres + "/";
	destination += "explicit=false/limit=10/rss.xml?partnerId=30&LS_PARAM=http%3A%2F%2Fclick.linksynergy.com%2Ffs-bin%2Fstat%3Fid%3Dl%2FaTOwnX308%26offerid%3D78941%26type%3D3%26subid%3D0%26tmpid%3D1826%26RD_PARM1%3D";
//	var pageData = url.fetch("http://localhost/top10songsiTunes.xml");
	print("Destination URL = " + destination);
	var pageData = url.fetch(destination);

	print("url.response = " + url.response);	
//	print("pageData = " + pageData);
	if(url.response == 0 || url.response > 400 || pageData == 0 || pageData == "Could not load URL")
	{
		print("Update list(): could not retrieve data!");
		return;
	}
	
	categories = new Array();
	categories.push("iTunes");
	
	listItems = new Array();
	listItems[0] = new Array();
	
	// Descend into the XML of the response to build our list
	var contentArr = new Array();
	contentArr['item'] = getNodeContent(pageData,"item");
	for (var idx in contentArr['item'])
	{
		var title = xmlDecode(getNodeContent(contentArr['item'][idx],"title", true));
		var link = xmlDecode(getNodeContent(contentArr['item'][idx],"link", true));
		var image = getNodeContent(contentArr['item'][idx],"itms:coverArt", true);
		listItems[0][idx] = new ListItem(title, link, image);
//		print("New list item: " + listItems[0][idx]);
	} 
	
	// Now that we're done searching for this category in the HTML, decode them
	for(idx = 0; idx < categories.length; idx++)
	{
		categories[idx] = xmlDecode(categories[idx]);
	}

	switchCategory();
	
}

/******************************************************************************
** Show the available categories and let the user select which category to
** display.
******************************************************************************/
function chooseCategory()
{
	var formFields = Array();
	var idx;
	
	formFields[0] = new FormField;
	formFields[0].name = 'category';
	formFields[0].type = 'popup';
	formFields[0].title = 'Category:';
	formFields[0].description = 'Choose a category to see its most popular auctions.';

	formFields[0].option = new Array();
	for(idx = 0; idx < categories.length; idx++)
	{
		formFields[0].option[idx] = categories[idx];
	}
	
	formResults = form(formFields, 'Choose a Category', 'OK');
	if (formResults != null) 
	{
		// Find the category in our list and show that one
		switchCategory(formResults[0]);
	}
}

/******************************************************************************
** Switch the category our widget is looking at (or switch to the first 
** category if the requested category cannot be found).
******************************************************************************/
function switchCategory(newCat)
{
	var idx;
	
	if(!newCat)
	{
		categoryIdx = 0;		
	}
	else
	{
		// Try to find the newCat string in our category list
		for(idx = 0; idx < categories.length; idx++)
		{
			if(newCat == categories[idx])
			{
				categoryIdx = idx;
				break;
			}
		}
	}
	
	// Update the buttons with the new display text
	for(idx = 0; idx < 10; idx++)
	{
		if(idx < listItems[categoryIdx].length)
			textArray[idx].data = listItems[categoryIdx][idx].display;
		else
			textArray[idx].data = "";
	}
		
	// Save the current category of choice
	preferences.curCategory.value = categories[categoryIdx];
}

/******************************************************************************
**
** Create the value of the music filter from all the genre preferences selected
**
******************************************************************************/
function buildFilter()
{
	var filterHi 	= 0x0;
	var filter 		= 0x0;
	
	if(preferences.genreBlues.value == 1)
		filterHi |= 0x00000080;
	if(preferences.genreChildrens.value == 1)
		filterHi |= 0x00000020;
	if(preferences.genreClassical.value == 1)
		filterHi |= 0x00000010;
	if(preferences.genreComedy.value == 1)
		filterHi |= 0x00000040;
	if(preferences.genreCountry.value == 1)
		filterHi |= 0x00000008;
	if(preferences.genreElectronic.value == 1)
		filterHi |= 0x00000004;
	if(preferences.genreHoliday.value == 1)
		filterHi |= 0x00000002;
	if(preferences.genreOpera.value == 1)
		filterHi |= 0x00000001;
		
	if(preferences.genreAlternative.value == 1)
		filter |= 0x00020000;
	if(preferences.genreAnime.value == 1)
		filter |= 0x00000008;
	if(preferences.genreAudiobooks.value == 1)
		filter |= 0x00001000;
	if(preferences.genreDance.value == 1)
		filter |= 0x00100000;
	if(preferences.genreDisney.value == 1)
		filter |= 0x00000400;
	if(preferences.genreEasyListening.value == 1)
		filter |= 0x00000040;
	if(preferences.genreEnka.value == 1)
		filter |= 0x00000010;
	if(preferences.genreFolk.value == 1)
		filter |= 0x08000000;
	if(preferences.genreFrenchPop.value == 1)
		filter |= 0x00000200;
	if(preferences.genreGermanFolk.value == 1)
		filter |= 0x00000080;
	if(preferences.genreGermanPop.value == 1)
		filter |= 0x00000100;
	if(preferences.genreHipHopRap.value == 1)
		filter |= 0x00080000;
	if(preferences.genreInspirational.value == 1)
		filter |= 0x00008000;
	if(preferences.genreJPop.value == 1)
		filter |= 0x00000020;
	if(preferences.genreJazz.value == 1)
		filter |= 0x04000000;
	if(preferences.genreKayokyoku.value == 1)
		filter |= 0x00000004;
	if(preferences.genreLatin.value == 1)
		filter |= 0x02000000;
	if(preferences.genreNewAge.value == 1)
		filter |= 0x01000000;
	if(preferences.genrePop.value == 1)
		filter |= 0x00800000;
	if(preferences.genreRnBSoul.value == 1)
		filter |= 0x00400000;
	if(preferences.genreReggae.value == 1)
		filter |= 0x00002000;
	if(preferences.genreRock.value == 1)
		filter |= 0x00010000;
	if(preferences.genreSoundtrack.value == 1)
		filter |= 0x00200000;
	if(preferences.genreSpokenWord.value == 1)
		filter |= 0x00000800;
	if(preferences.genreVocal.value == 1)
		filter |= 0x00004000;
	if(preferences.genreWorld.value == 1)
		filter |= 0x00040000;
		
	var filterStr = filter.toString(16);
	while(filterStr.length < 7)
		filterStr = "0" + filterStr;
	var filterHiStr = filterHi.toString(16);
	while(filterHiStr.length < 2)
		filterHiStr = "0" + filterHiStr;
		
	print("\n\nFilter result = " + filterHiStr + filterStr + "\n\n");
			
	return filterHiStr + filterStr;
}

/******************************************************************************
**
** Begin ListItem object definition
**
******************************************************************************/

/******************************************************************************
** Create a new ListItem object
******************************************************************************/
function ListItem(name, link, image, idx)
{
	this.link 			= link;	
	this.display	 	= name;
	this.image			= image
}

/*****************************************************************************
** Print a string representation of the dynamic ListItem elements
******************************************************************************/
ListItem.prototype.toString = function()
{
	var str = this.display + ";;" + this.link + ";;" + this.image + ";;";
		
	return str;		
}

/******************************************************************************
** App wrapper to call the appropriate ListItem's onMouseEnter
******************************************************************************/
function listItemMouseEnter(itemIdx)
{
	buttonArray[itemIdx].src = "Resources/blackButton2.png";
}

/******************************************************************************
** App wrapper to call the appropriate ListItem's onMouseExit
******************************************************************************/
function listItemMouseExit(itemIdx)
{
	buttonArray[itemIdx].src = "Resources/blackButton.png";
}

/******************************************************************************
** App wrapper to call the appropriate ListItem's onMouseUp
******************************************************************************/
function listItemMouseUp(itemIdx)
{
//	printListItems();
	print("listItemMouseUp: categoryIdx = " + categoryIdx + ", itemIdx = " + itemIdx);
	print("Current listItem is: " + listItems[categoryIdx][itemIdx]);
	openURL(listItems[categoryIdx][itemIdx].link);
	
}
